/*
File: TangoSimModule.java
Date		Author		Changes
Feb 14 97	Lukasz Beca	Created

*/


package tango.simulator;

import messages.AppEventMessage;
import consts.Const;
import tango.simulator.SimModule;
import threads.SimThreadIn;
import threads.ThreadOut;
import java.net.*;
import java.io.*;

public abstract class TangoSimModule implements SimModule {

  
  private SimThreadIn tango;
  private String host;
  private int port;
  private int AT;
  private String name;
  private boolean joined;


  public TangoSimModule() {

    host = null;
    port = 0;
    AT = 0;
    name = null;
    joined = false;


  }



 
  
  private void run() throws IOException {

    Socket s;

    System.out.println("TangoSimModule: Simulation started ");

    s = new Socket(host, port);

    handleConnection(s);
       
    System.out.println("TangoSimModule: Connected to " + host + " " + port);
    
    tango.sendLOGIN(name);

    delay();

    if (!joined) {
      tango.sendJOIN(AT);
    }

  }

  public void setJoined() {


    joined = true;

  }

  public boolean joined() {

    return joined;

  }

   private void handleConnection(Socket s) {

    ThreadOut o;
      
    System.out.println("TangoSimModule: Connection accepted by remote host, starting threads ......");
    o = new ThreadOut(s);   
    tango = new SimThreadIn(o, s, AT, this);    
        
  } 


  

  /********************* used by simulation engine ******************/

  public abstract boolean accepts(String id);


  public abstract void processSimEvent(SimEvent evt);


  public abstract void receive(AppEventMessage mesg);


  public abstract void init(String host,int port,SimEventConsumer sec);


  protected void send(AppEventMessage mesg) {

    tango.sendMessage(mesg);

  }
  

  public void start(String hostpar, int portpar, int ATpar, String namepar) throws IOException {

    host = hostpar;
    port = portpar;
    AT = ATpar;
    name = new String(namepar);
 
    run();

  }


  public synchronized void delay() {

    try {
      wait(5000);
    } catch(Exception e) {
      
      System.out.println("TangoSimModule:" + e);

    }

  }


  public void logout() {

    tango.sendLOGOUT();

  }



  /*********************************************************************/


}














